@@ -117,7 +117,7 @@ Unless overridden by a more specific face association."
 
 (defface lsp-face-semhl-string
   '((t (:inherit font-lock-string-face)))
-  "Face used for keywords."
+  "Face used for strings."
   :group 'lsp-semantic-tokens)
 
 (defface lsp-face-semhl-number
@@ -135,11 +135,6 @@ Unless overridden by a more specific face association."
   "Face used for operators."
   :group 'lsp-semantic-tokens)
 
-(defface lsp-face-semhl-namespace
-  '((t (:inherit font-lock-keyword-face)))
-  "Face used for namespaces."
-  :group 'lsp-semantic-tokens)
-
 (defface lsp-face-semhl-type
   '((t (:inherit font-lock-type-face)))
   "Face used for types."
@@ -192,11 +187,6 @@ Unless overridden by a more specific face association."
   "Face used for macros."
   :group 'lsp-semantic-tokens)
 
-(defface lsp-face-semhl-variable
-  '((t (:inherit font-lock-variable-name-face)))
-  "Face used for variables."
-  :group 'lsp-semantic-tokens)
-
 (defface lsp-face-semhl-parameter
   '((t (:inherit font-lock-variable-name-face)))
   "Face used for parameters."
